"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMessageConnection = void 0;
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
const reader_1 = require("./reader");
const writer_1 = require("./writer");
/**
 * Create VSCode JSON RPC `MessageConnection` over WebSocket.
 * @param webSocket
 * @param logger
 */
exports.createMessageConnection = (webSocket, logger = createConsoleLogger()) => new Promise((onConnection) => {
    webSocket.onopen = () => {
        const conn = vscode_jsonrpc_1.createMessageConnection(reader_1.createMessageReader(webSocket), writer_1.createMessageWriter(webSocket), logger);
        conn.onClose(() => conn.dispose());
        onConnection(conn);
    };
});
const createConsoleLogger = () => ({
    error: (message) => console.error(message),
    warn: (message) => console.warn(message),
    info: (message) => console.info(message),
    log: (message) => console.log(message),
});
//# sourceMappingURL=connection.js.map