"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMessageReader = void 0;
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
exports.createMessageReader = (wsocket) => {
    let state = "initial";
    const buffer = [];
    const errorEmitter = new vscode_jsonrpc_1.Emitter();
    const closeEmitter = new vscode_jsonrpc_1.Emitter();
    // This is not used, but required by the interface.
    const partialMessageEmitter = new vscode_jsonrpc_1.Emitter();
    const messageEmitter = new vscode_jsonrpc_1.Emitter();
    const readMessage = (message) => {
        if (state === "initial") {
            buffer.push({ message });
        }
        else if (state === "listening") {
            messageEmitter.fire(JSON.parse(message));
        }
    };
    const fireError = (error) => {
        if (state === "initial") {
            buffer.push({ error });
        }
        else if (state === "listening") {
            errorEmitter.fire(error);
        }
    };
    const fireClose = () => {
        if (state === "listening")
            closeEmitter.fire();
        state = "closed";
    };
    const messageHandler = (event) => readMessage(event.data);
    const closeHandler = (event) => {
        // Note that error code is almost always generic 1006 (closed abnormally)
        if (event.code !== 1000) {
            fireError(new Error(`WebSocket Error (${event.code}): ${event.reason}`));
        }
        fireClose();
    };
    wsocket.addEventListener("message", messageHandler);
    // Note that we're not adding "errror" event handler because WebSocket doesn't pass
    // anything useful there. "close" event may receive error details.
    wsocket.addEventListener("close", closeHandler);
    return {
        get onError() {
            return errorEmitter.event;
        },
        get onClose() {
            return closeEmitter.event;
        },
        get onPartialMessage() {
            return partialMessageEmitter.event;
        },
        listen(callback) {
            if (state !== "initial")
                throw new Error(`Already listening`);
            const res = messageEmitter.event(callback);
            state = "listening";
            if (buffer.length === 0)
                return res;
            for (const event of buffer) {
                if (event.message) {
                    readMessage(event.message);
                }
                else if (event.error) {
                    fireError(event.error);
                }
            }
            buffer.length = 0;
            return res;
        },
        dispose() {
            wsocket.removeEventListener("message", messageHandler);
            wsocket.removeEventListener("close", closeHandler);
            errorEmitter.dispose();
            closeEmitter.dispose();
            partialMessageEmitter.dispose();
            buffer.length = 0;
        },
    };
};
//# sourceMappingURL=reader.js.map