import { Emitter } from "vscode-jsonrpc";
export const createMessageWriter = (wsocket) => {
    const errorEmitter = new Emitter();
    const closeEmitter = new Emitter();
    const closeHandler = () => {
        closeEmitter.fire();
    };
    wsocket.addEventListener("close", closeHandler);
    let errorCount = 0;
    return {
        get onError() {
            return errorEmitter.event;
        },
        get onClose() {
            return closeEmitter.event;
        },
        async write(msg) {
            try {
                // Can throw InvalidStateError if this was called when readyState is CONNECTING.
                wsocket.send(JSON.stringify(msg));
            }
            catch (e) {
                errorEmitter.fire([toError(e), msg, ++errorCount]);
            }
        },
        end() { },
        dispose() {
            errorEmitter.dispose();
            closeEmitter.dispose();
            wsocket.removeEventListener("close", closeHandler);
        },
    };
};
const toError = (err) => err instanceof Error ? err : new Error((err === null || err === void 0 ? void 0 : err.message) || "Unknown error");
//# sourceMappingURL=writer.js.map